import redis


class RedisCache:
    topic = None
    redis_client = None

    def __init__(self, hostname, key, topic):
        self.topic = topic
        self.redis_client = redis.StrictRedis(host=hostname,
                                              port=6380,
                                              password=key,
                                              ssl=True, ssl_cert_reqs=u'none')
        if not self.redis_client.ping():
            raise Exception("We can't connect to the redis server, please check the config")

    def get_offset(self):
        offset = self.redis_client.get(self.topic)
        if offset is None:
            return "-1"
        else:
            return int(offset)

    def set_offset(self, sequence_number):
        self.redis_client.set(self.topic, sequence_number)



from azure.eventhub import EventHubClient, Offset

from .redis_cache import RedisCache


class Consumer:
    consumer_group = None
    eventhubs_client = None
    offset = Offset("-1")
    redis_cache = None

    def __init__(self, eventhub, address, user, key,
                 consumer_group, redis_hostname, redis_key):
        self.consumer_group = consumer_group
        self.eventhubs_client = EventHubClient(address,
                                               debug=False,
                                               username=user,
                                               password=key)

        redis_topic = f"eventhubs-{eventhub}-{consumer_group}"
        self.redis_cache = RedisCache(redis_hostname, redis_key, redis_topic)

    def recieve(self):
        OFFSET = Offset(self.redis_cache.get_offset())
        receiver = self.eventhubs_client.add_receiver(self.consumer_group,
                                                      "0",
                                                      prefetch=5000,
                                                      offset=OFFSET)
        self.eventhubs_client.run()
        messages = receiver.receive(timeout=100)
        self.eventhubs_client.stop()
        return messages

    def commit(self, event_data):
        self.redis_cache.set_offset(event_data.sequence_number)



from azure.eventhub import EventHubClient, EventData


class Producer:
    client = None
    sender = None

    def __init__(self, address, user, key):
        self.client = EventHubClient(address,
                                     debug=False,
                                     username=user,
                                     password=key)
        self.sender = self.client.add_sender(partition="0")
        self.client.run()

    def send(self, msg):
        self.sender.send(EventData(str(msg)))

    def stop(self):
        self.client.stop()



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='azure_eventhubs_client',
    version="0.91",
    author="Knockri",
    author_email="saman@knockri.com",
    url='https://github.com/KnockriInc/azure_eventhubs_client',
    maintainer='Saman A. Pour',
    maintainer_email='samanamp@knockri.com',
    description="Azure Eventhubs client that simply works",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=['redis', 'azure-eventhub'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)
